--Honk, version 1.3, by Pizzasgood (Rin Ganborro)
--This plugin allows you to honk your ship's horn with the /honk command
--or by simply emoting 'honks' in Sector chat, ala "T/me honks".
--It works by watching Sector chat for a player to emote "honks", and
--then playing an appropriate audio file based on their ship and their
--distance from you.  My primary motivation for writing this was that
--the Behemoth XC doesn't have any weapons ports, making it a terrebly
--boring ship to fly (unless some noob gets in your way and pulls the
--deer-in-headlights routine :D).  Besides, even space-trucks should
--have horns, to /dev/null with physics.  Ours aren't very real anyway.
--
--Keep in mind that only other players who have this plugin will be able
--to hear you when you honk.  Everybody else only sees the emote.
--
declare('honk', honk or {})
--math.randomseed( os.time() ); --seed the random number generate (this isn't needed right now, but maybe later)

-- SETTINGS
local max_vol = 1.0;
local min_vol = 0.1;

local distance_factor = 500; --smaller numbers attenuate faster (this is not currently used, but would be if you used curvey attenuation)
local max_distance = 5000; --distance to use for ships out of radar range, if only_in_range not set to true
local only_in_range = false; --when true, don't play audio for honks that happen outside radar range
-- END OF SETTINGS

local honk_message = "honks" --DO NOT CHANGE, or it won't work with other users.  This is how it detects that a player has honked.

--Define names (or partial names) of ships for which honks exist
--The second entry is the folder that has the appropriate honk.ogg file
--Place more specific entries toward the beginning and more general entries toward the bottom.
--Remember to escape special symbols like - with a %
local ship_classes = {
	{"Atlas", "Atlas"},
	{"Behemoth", "Behemoth"},
	{"Centaur", "Centaur"},
	{"Centurion", "Centurion"},
	{"EC%-88", "EC-88"},
	{"EC", "EC"},
	{"Hornet", "Hornet"},
	{"Marauder", "Marauder"},
	{"Prometheus", "Prometheus"},
	{"Ragnarok", "Ragnarok"},
	{"Raptor", "Raptor"},
	{"Revenant", "Revenant"},
	{"Valkyrie", "Valkyrie"},
	{"Vult", "Vulture"},
	{"Greyhound", "Warthog"},
	{"Warthog", "Warthog"},
	{"Wraith", "Wraith"},
};

--Load the sounds into the system
local audio_path = "plugins/honk/audio/"
for index,ship_class in pairs(ship_classes) do
	gksound.GKLoadSound{soundname=ship_class[1].."_honk", filename=audio_path..ship_class[2].."/honk.ogg"};
end

--This simply sends the honk to sector chat, so you don't have to manually emote it
--Just bind 'honk' to the key you want to use as your ship's horn.
--TODO:  Have this optionally auto-bound to the weapon controls if you're in an XC (or maybe to weapon controls that don't have weapons attached regardless of ship?)
--This also accepts the 'test' option followed by a distance in meters.  Your own horn will be honked locally, without others hearing it, and will sound like it came from the distance specified.
--You can also specify a ship name after the distance, and the horn will be of the type used by that ship.
function honk.honk(_,data)
	if (data ~= nil and data[1] == "test") then
		local data_param = {};
		data_param.msg = honk_message;
		data_param.name = GetPlayerName();
		data_param.dist = data[2];
		if (#data > 2) then
			data_param.ship = data[3];
		end
		honk.chat_processor:CHAT_MSG_SECTOR_EMOTE(nil, data_param);
	else
		SendChat("/me "..honk_message, "SECTOR");
	end
end
RegisterUserCommand("honk",honk.honk);

--This function looks at each sector-chat emote to detect honks.
--When a honk is detected, it determines which audio to play and
--and what volume to use - if it should play at all.
honk.chat_processor = {};
function honk.chat_processor:CHAT_MSG_SECTOR_EMOTE(event, data)
	--data has 'msg', 'name', and 'faction'
	if (data.msg == honk_message) then
		local my_id = GetCharacterIDByName(GetPlayerName());
		local honker_id = GetCharacterIDByName(data.name);
		local ship = GetPrimaryShipNameOfPlayer(honker_id);
		if (data.ship ~= nil) then
			ship = data.ship;
		end
		local distance;
		if (data.dist ~= nil) then
			distance = data.dist;
		elseif (my_id == honker_id) then
			distance = 0;
		else
			distance = GetPlayerDistance(honker_id);
		end
		if (not only_in_range and distance == nil) then
			distance = max_distance;
		end
		if (distance ~= nil and ship ~= nil) then
			--compute the volume based on distance
			local volume = (max_vol - min_vol) / (distance/distance_factor + 1) + min_vol; --curvey attenuation
			--local volume = ((min_vol-max_vol)/max_distance)*distance + max_vol; --linear attenuation
			
			--decide which sound to play
			--TODO:  Find a fog-horn clip and a way to determine if we're inside a cappy
			--TODO:  Add several versions of each ship's honk to add variety (probably too lazy to bother)
			for index,ship_class in pairs(ship_classes) do
				if (string.find(ship, ship_class[1]) ~= nil) then
					gksound.GKPlaySound(ship_class[1].."_honk", volume);
					break;
				end
			end
		end
	end
end
RegisterEvent(honk.chat_processor, "CHAT_MSG_SECTOR_EMOTE");
